#include <cstdio>
#include <cstring>
#include <algorithm>
#include <vector>

using namespace std;

const int MAX = 50005;

struct Edge {
    int to, limit;
    Edge(int to_ = 0, int limit_ = 0) : to(to_), limit(limit_) {}
};

int n, m, k;
vector <Edge> v[MAX];
bool used[MAX];

int reachable(int node, int limit, int blocked) {
    int ret = 1;
    used[node] = true;
    for (int i = 0; i < (int)v[node].size(); i++)
        if (v[node][i].to != blocked && !used[v[node][i].to] && limit < v[node][i].limit)
            ret += reachable(v[node][i].to, limit, blocked);
    return ret;
}

bool connected(int snow) {
    memset(used, 0, sizeof(used));
    return reachable(0, snow, -1) == n;
}


vector <int> artPoints;
int enter[MAX], lowest[MAX], id;

void artRecurse(int node, int parent, int limit) {
    int numChildren = 0;
    enter[node] = lowest[node] = ++id;
    bool isArticulation = false;
    for (int i = 0; i < (int)v[node].size(); i++) {
        if (v[node][i].limit > limit) {
            if (!enter[v[node][i].to]) {
                numChildren++;
                artRecurse(v[node][i].to, node, limit);
                lowest[node] = min(lowest[node], lowest[v[node][i].to]);
                if (parent != -1 && lowest[v[node][i].to] >= enter[node])
                    isArticulation = true;
            } else if (v[node][i].to != parent)
                lowest[node] = min(lowest[node], enter[v[node][i].to]);
        }
    }
    if (isArticulation || (parent == -1 && numChildren > 1))
        artPoints.push_back(node);
}

vector <int> getDangerous(int snow) {
    id = 0;
	artPoints.clear();
	memset(enter, 0, sizeof(enter));
	
	for (int node = 0; node < n; node++)
		if (!enter[node]) artRecurse(node, -1, snow);
	sort(artPoints.begin(), artPoints.end());
    return artPoints;
}

pair <int, vector <int>> smarty() {
    int left = 0, right = 1000000001;
    while (left <= right) {
        int mid = (left + right) / 2;
        if (!connected(mid)) {
            right = mid - 1;
            continue;
        }
        vector <int> dangerous = getDangerous(mid);
        (int)dangerous.size() < k ? left = mid + 1 : right = mid - 1;
    }
    if (!connected(right + 1)) {
        return make_pair(-1, vector<int>());
    }
    return make_pair(right + 1, getDangerous(right + 1));
}

int main(void) {
    scanf("%d %d %d", &n, &m, &k);
    for (int i = 0; i < m; i++) {
        int node1, node2, limit;
        scanf("%d %d %d", &node1, &node2, &limit);
        node1--, node2--;
        v[node1].push_back(Edge(node2, limit));
        v[node2].push_back(Edge(node1, limit));
    }
 
    pair <int, vector<int>> ans = smarty();
    printf("%d %d\n", ans.first, (int)ans.second.size());
    for (int i = 0; i < (int)ans.second.size(); i++)
        printf("%d%c", ans.second[i] + 1, i + 1 == (int)ans.second.size() ? '\n' : ' ');

    return 0;
}
