#include <cstdio>
#include <cstring>
#include <vector>

using namespace std;

const int MAX = 50005;

struct Edge {
    int to, limit;
    Edge(int to_ = 0, int limit_ = 0) : to(to_), limit(limit_) {}
};

int n, m, k;
vector <Edge> v[MAX];
bool used[MAX];

int reachable(int node, int limit, int blocked) {
    int ret = 1;
    used[node] = true;
    for (int i = 0; i < (int)v[node].size(); i++)
        if (v[node][i].to != blocked && !used[v[node][i].to] && limit < v[node][i].limit)
            ret += reachable(v[node][i].to, limit, blocked);
    return ret;
}

bool connected(int snow) {
    memset(used, 0, sizeof(used));
    return reachable(0, snow, -1) == n;
}

vector <int> getDangerousSlow(int snow) {
    vector <int> dangerous;
    for (int blocked = 0; blocked < n; blocked++) {
        memset(used, 0, sizeof(used));
        if (reachable(!blocked, snow, blocked) < n - 1)
            dangerous.push_back(blocked);
    }
    return dangerous;
}

pair <int, vector <int>> medium() {
    int left = 0, right = 1000000001;
    while (left <= right) {
        int mid = (left + right) / 2;
        if (!connected(mid)) {
            right = mid - 1;
            continue;
        }
        vector <int> dangerous = getDangerousSlow(mid);
        (int)dangerous.size() < k ? left = mid + 1 : right = mid - 1;
    }
    if (!connected(right + 1)) {
        return make_pair(-1, vector<int>());
    }
    return make_pair(right + 1, getDangerousSlow(right + 1));
}

int main(void) {
    scanf("%d %d %d", &n, &m, &k);
    for (int i = 0; i < m; i++) {
        int node1, node2, limit;
        scanf("%d %d %d", &node1, &node2, &limit);
        node1--, node2--;
        v[node1].push_back(Edge(node2, limit));
        v[node2].push_back(Edge(node1, limit));
    }
 
    pair <int, vector<int>> ans = medium();
    printf("%d %d\n", ans.first, (int)ans.second.size());
    for (int i = 0; i < (int)ans.second.size(); i++)
        printf("%d%c", ans.second[i] + 1, i + 1 == (int)ans.second.size() ? '\n' : ' ');

    return 0;
}
